#ifndef _AEROTAP_SDK_H_
#define _AEROTAP_SDK_H_

#pragma once

#include <vector>

//  nMode = 0:8bit Depth,  1:Point Cloud, 2 14Bit Depth, 3: Point Cloud 14Bit
//          10: Pass Thru 8 Bit;
//          11: Pass Thru 11 Bit;
//          12: Pass Thru 14 Bit;
/// typedef BOOL(__cdecl *InitAeroCam)(HWND, UINT, int, int, void *pColor, void *pColorOrg, void *pDepth, int nMode);

enum DEPTHTYPE
{
	AERO_8BIT = 0,
	AERO_8BIT_POINT,
	AERO_14BIT,
	AERO_14BIT_POINT,
	AERO_THRU_8BIT = 10,
	AERO_THRU_11BIT,
	AERO_THRU_14BIT
};
enum IMAGETYPE // actions on mouse Z
{
	IMAGE_ORIGINAL = 0,
	IMAGE_GRAY,
	IMAGE_HISTGRAM,
	IMAGE_RESIZED,
	IMAGE_LASTDIFF,
	IMAGE_WIDE,
	IMAGE_DEPTH,
	IMAGE_RAW,
	IMAGE_DEPTH_RAW,
	IMAGE_DEPTH_DISTANCE
};

enum {
	_CURSOR_FAR = 0,
	_CURSOR_MOVING,
	_CURSOR_STOPPING,
	_CURSOR_STOP,
	_CURSOR_CLICK,

};


#ifndef POINT3I
typedef struct _POINT3I
{
	int x;
	int y;
	int z;
} POINT3I;
#endif

#ifndef POINT3F
typedef struct _POINT3D
{
	float x;
	float y;
	float z;
} POINT3F;
#endif

#ifndef POINT2F
typedef struct _POINT2D
{
	float x;
	float y;
} POINT2F;
#endif

#ifndef ObjPeak
typedef struct _ObjectPeak
{
	POINT3F pos[2];	 // Center
	POINT3F posVal[2];	 // Center in mm
	POINT3F posG[2];	 // Center of Gravity
	int nCount[2];  // Pixel number
	int nAction[2];	// Moving/Stop/Click
	int nDetection[2];
	DWORD clLive[2];
	RECT rectArea[2];
	RECT rect[2];
	int nGray[2];
	BOOL bNear;
} ObjPeak;
#endif

typedef struct _ObjectDepth
{
	int d;			// depth
	POINT pos;		// x,y
	int l;			// lenght
	int g;			// group
	int gray;		// grayscale
	unsigned char rgb[3];
	int nVal;		//Real Depth
	POINT3F posVal;	//x,y in mm ( center 0,0 )
	int nType;
	int p;			// point
	int nCount;

} ObjDepth;

typedef struct _MyVoxel
{
	POINT3F grid;
	POINT3F point;
	POINT pos2d;
	int id;
	int count;
	bool bValid;
	unsigned char rgb[3];
	unsigned char gray;
} MyVoxel;

typedef struct _POINTCLOUD
{
	int nMaxPoints;
	int nMaxVoxel;
	int nScale;
	int nPoints;
	int nVoxels;
	ObjDepth *points;
	MyVoxel *grids;
	POINT posCenter;
	RECT rectArea;
	bool bReady;
	float nFocalLengthW;
	float nFocalLengthH;

} POINTCLOUD;

typedef struct _aeroinfo
{
	int nDepthBits;
	int nDevicePID;
} AEROINFO;

//  nMode = 0:8bit Depth,  1: Point Cloud, 2 14Bit Depth, 3: Point Cloud 14Bit
//          10: Pass Thru 8 Bit;
//          11: Pass Thru 11 Bit;
//          12: Pass Thru 14 Bit;

typedef BOOL (__cdecl *InitAeroCam)(HWND, UINT, int, int,void *pColor, void *pColorOrg, void *pDepth,int nMode);
typedef void (__cdecl *ExitAeroCam)(void);
typedef BOOL (__cdecl *InitAeroStereoCam)( HWND hParentWnd, UINT nMSG,int nDeviceNo, UINT nFPS, void *pColor,int nWidth, int nHeight,BOOL bMJPG  );

typedef int (__cdecl *AERO_IsCameraConnected)();
typedef BOOL (__cdecl *AERO_StartCam)(int nDevice);
typedef void (__cdecl *AERO_SwitchDevice)(int nDevice);
typedef int (__cdecl *AERO_GetFPS)();
typedef void (__cdecl *AERO_StopCam)(int nDevice);
typedef void (__cdecl *AERO_SetViewArea)(RECT *);
typedef void (__cdecl *AERO_SetImageMode)(int mode, int x, int y);
typedef void (__cdecl *AERO_SetColorMode)(bool mode);
typedef int (__cdecl *AERO_SetZoom)(int z);
typedef BOOL (__cdecl *AERO_GetImage)(BITMAPINFOHEADER * pBuffer,int nType,int *nMax/*=NULL*/, int *nTotal/*=NULL*/,int *nCount/*=NULL*/, int *nFrame/*=NULL*/, int *nP/*=NULL*/, int *nBlackout/*=NULL*/);
typedef void (__cdecl *AERO_SetDepthRange)(int min,int max);
typedef void (__cdecl *AERO_SetROI)(RECT *pRect);
typedef void (__cdecl *AERO_EnableROIFilter)(int nMode);
typedef void (__cdecl *AERO_SetEnableNear)(int mode);
typedef void (__cdecl *AERO_LowTexture)(bool mode);
typedef BOOL (__cdecl *AERO_SetResolution)(int nDevice,int nWidth, int nHeight);
// mode 0: VGA, mode 1:QVGA, mode 2: WVGA
typedef int (__cdecl *AERO_LoadZDTable)(int *pZDTable, int mode);
typedef void (__cdecl *AERO_DoAdjust)();
typedef BOOL (__cdecl *AERO_IsNewFrame)(int nDevice);
typedef BOOL (__cdecl *AERO_IsNearObject)();
typedef void* (__cdecl *AERO_GetTracker)();
typedef void (__cdecl *AERO_EnableGrayscale)(bool mode);
typedef void (__cdecl *AERO_DoHandDetect)(int mode);
typedef void (__cdecl *AERO_DoFaceDetect)(int mode);
typedef void* (__cdecl *AERO_GetGrayImage)(void *pGray,bool mode,RECT *pRect, WORD *_pDepth,int nThreshold);
typedef void* (__cdecl *AERO_GetColorDepthImage)(void *pColor, void *pDepth , bool mode,RECT *pCrop);
typedef void* (__cdecl *AERO_GetColorImage)(void *_pColor, bool mode,RECT *pCrop);
typedef int (__cdecl *AERO_GetPeakPos)(void *);
typedef void (__cdecl *AERO_SetDebugWnd)(void *);
typedef void (__cdecl *AERO_SetHandMode)(int m);
typedef void (__cdecl *AERO_Set2HandMode)(bool m);
typedef void (__cdecl *AERO_SetSideInMode)(bool m);
typedef void (__cdecl *AERO_GetZoomArea)(RECT *pRect);
typedef void (__cdecl *AERO_EnableZoomAdjust)(bool m);
typedef void (__cdecl *AERO_SetSavingMode)(int nMode);
typedef void (__cdecl *AERO_SetAutoAdjust)(bool bMode);
typedef void (__cdecl *AERO_SetZoomArea)(RECT *rect);
typedef void (__cdecl *AERO_EnableSavePositive)(bool bMode);
typedef void (__cdecl *AERO_SetScale)(float s);
typedef void (__cdecl *AERO_SetEnableGesture)(BOOL mode);
typedef void (__cdecl *AERO_SetGestureCallback)(HWND hWnd, UINT nMSG, UINT nMASK);
typedef void (__cdecl *AERO_GetHandObjects)(void *pDResultLast);
typedef void (__cdecl *AERO_GetFaceObjects)(void *pDResultLast);
typedef void (__cdecl *AERO_SetNearRange)(int n);
typedef int (__cdecl *AERO_GetObject)(RECT *pRect, int size);
typedef unsigned char * (__cdecl *AERO_GetSmallImage)(int *w,int *h);
typedef WORD * (__cdecl *AERO_GetSmallImageW)(int *w, int *h);
typedef int (__cdecl *AERO_EnableDepthLog)(bool bMode);

typedef void (__cdecl *AERO_GetSDKVersion)(char *szVer  );
typedef void (__cdecl *AERO_GetFWVersion)(char *szVer  );
typedef void (__cdecl *AERO_GetSN)(char *szSN  );
typedef void (__cdecl *AERO_GetPID)(char *szPID  );

typedef void(__cdecl *AERO_GetDeviceInfo)(int nDevice, AEROINFO *pInfo);
typedef void(__cdecl *AERO_GetFocalLength)(int nDevice, float *w, float *h);
typedef void(__cdecl *AERO_SetFocalLength)(int nDevice, float w, float h);

typedef int(__cdecl *AERO_SetIRMode)(int nDevice, WORD nValue);
typedef int(__cdecl *AERO_GetPType)();

typedef int (__cdecl *AERO_GetDeviceList)(void **pDeviceList);

typedef struct _DetectionResult
{
	RECT rect;
	RECT rectArea;
	BOOL bUsed;
	int nCount;
	POINT3I pCenter;
	DWORD clock;
	DWORD clockStart;
	DWORD clockStop;
	int nObjectType;
	int nDetected;
	int nMovedCount;
	int nMoving;
	POINT pDelta;
	RECT rectGesture;
	int nEffectType;
} DETECTIONRESULT;


/*
InitAeroCam g_pInitAeroCam = NULL;
ExitAeroCam g_pExitAeroCam = NULL;
InitAeroStereoCam g_pInitAeroStereoCam = NULL;

AERO_IsCameraConnected g_pAERO_IsCameraConnected = NULL;
AERO_StartCam g_pAERO_StartCam = NULL;
AERO_StopCam g_pAERO_StopCam = NULL;
AERO_GetFPS g_pAERO_GetFPS = NULL;
AERO_SetViewArea g_pAERO_SetViewArea = NULL;
AERO_SetImageMode g_pAERO_SetImageMode = NULL;
AERO_SetColorMode g_pAERO_SetColorMode = NULL;
AERO_SetZoom g_pAERO_SetZoom = NULL;
AERO_GetImage g_pAERO_GetImage = NULL;
AERO_SetDepthRange g_pAERO_SetDepthRange = NULL;
AERO_IsNearObject g_pAERO_IsNearObject = NULL;
AERO_SetROI g_pAERO_SetROI = NULL;
AERO_EnableROIFilter g_pAERO_EnableROIFilter = NULL;
AERO_SetEnableNear g_pAERO_SetEnableNear = NULL;
AERO_LowTexture g_pAERO_LowTexture = NULL;
AERO_SetResolution g_pAERO_SetResolution = NULL;
AERO_LoadZDTable g_pAERO_LoadZDTable = NULL;
AERO_DoAdjust g_pAERO_DoAdjust = NULL;
AERO_IsNewFrame g_pAERO_IsNewFrame = NULL;
AERO_GetTracker g_pAERO_GetTracker = NULL;
AERO_EnableGrayscale g_pAERO_EnableGrayscale = NULL;
AERO_DoHandDetect g_pAERO_DoHandDetect = NULL;
AERO_DoFaceDetect g_pAERO_DoFaceDetect = NULL;
AERO_GetGrayImage g_pAERO_GetGrayImage = NULL;
AERO_GetColorDepthImage g_pAERO_GetColorDepthImage = NULL;
AERO_GetColorImage g_pAERO_GetColorImage = NULL;
AERO_SetDebugWnd g_pAERO_SetDebugWnd = NULL;
AERO_SetHandMode g_pAERO_SetHandMode = NULL;
AERO_Set2HandMode g_pAERO_Set2HandMode = NULL;
AERO_SetSideInMode g_pAERO_SetSideInMode = NULL;
AERO_GetZoomArea g_pAERO_GetZoomArea = NULL;
AERO_EnableZoomAdjust g_pAERO_EnableZoomAdjust = NULL;
AERO_SetSavingMode g_pAERO_SetSavingMode = NULL;
AERO_SetAutoAdjust g_pAERO_SetAutoAdjust = NULL;
AERO_SetZoomArea g_pAERO_SetZoomArea = NULL;
AERO_EnableSavePositive g_pAERO_EnableSavePositive = NULL;
AERO_SetScale g_pAERO_SetScale = NULL;
AERO_SetEnableGesture g_pAERO_SetEnableGesture = NULL;
AERO_SetGestureCallback g_pAERO_SetGestureCallback = NULL;
AERO_GetHandObjects g_pAERO_GetHandObjects = NULL;
AERO_GetFaceObjects g_pAERO_GetFaceObjects = NULL;
AERO_SetNearRange  g_pAERO_SetNearRange = NULL;
AERO_GetPeakPos g_pAERO_GetPeakPos = NULL;
AERO_GetObject g_pAERO_GetObject = NULL;
AERO_GetSmallImage g_pAERO_GetSmallImage = NULL;
AERO_GetSmallImageW g_pAERO_GetSmallImageW = NULL;
AERO_EnableDepthLog g_pAERO_EnableDepthLog = NULL;

AERO_GetSDKVersion g_pAERO_GetSDKVersion = NULL;
AERO_GetFWVersion g_pAERO_GetFWVersion = NULL;
AERO_GetSN g_pAERO_GetSN = NULL;
AERO_GetPID g_pAERO_GetPID = NULL;

		g_pInitAeroCam = (InitAeroCam) GetProcAddress(m_hAeroTAPSDK, "InitAeroCam");
		g_pExitAeroCam = (ExitAeroCam) GetProcAddress(m_hAeroTAPSDK, "ExitAeroCam");
		g_pInitAeroStereoCam =  (InitAeroStereoCam) GetProcAddress(m_hAeroTAPSDK, "InitAeroStereoCam");

		g_pAERO_IsCameraConnected = (AERO_IsCameraConnected) GetProcAddress(m_hAeroTAPSDK, "AERO_IsCameraConnected");
		g_pAERO_StartCam = (AERO_StartCam) GetProcAddress(m_hAeroTAPSDK, "AERO_StartCam");
		g_pAERO_StopCam = (AERO_StopCam) GetProcAddress(m_hAeroTAPSDK, "AERO_StopCam");
		g_pAERO_SetViewArea = (AERO_SetViewArea) GetProcAddress(m_hAeroTAPSDK, "AERO_SetViewArea");
		g_pAERO_SetImageMode = (AERO_SetImageMode) GetProcAddress(m_hAeroTAPSDK, "AERO_SetImageMode");
		g_pAERO_SetColorMode = (AERO_SetColorMode) GetProcAddress(m_hAeroTAPSDK, "AERO_SetColorMode");
		g_pAERO_GetFPS = (AERO_GetFPS) GetProcAddress(m_hAeroTAPSDK, "AERO_GetFPS");
		g_pAERO_SetZoom = (AERO_SetZoom) GetProcAddress(m_hAeroTAPSDK, "AERO_SetZoom");
		g_pAERO_GetImage = (AERO_GetImage) GetProcAddress(m_hAeroTAPSDK, "AERO_GetImage");
		g_pAERO_SetDepthRange = (AERO_SetDepthRange) GetProcAddress(m_hAeroTAPSDK, "AERO_SetDepthRange");
		g_pAERO_SetROI = (AERO_SetROI) GetProcAddress(m_hAeroTAPSDK, "AERO_SetROI");
		g_pAERO_SetZoomArea = (AERO_SetZoomArea) GetProcAddress(m_hAeroTAPSDK, "AERO_SetZoomArea");
		g_pAERO_EnableROIFilter = (AERO_EnableROIFilter) GetProcAddress(m_hAeroTAPSDK, "AERO_EnableROIFilter");
		g_pAERO_LowTexture = (AERO_LowTexture) GetProcAddress(m_hAeroTAPSDK, "AERO_LowTexture");
		g_pAERO_SetResolution = (AERO_SetResolution) GetProcAddress(m_hAeroTAPSDK, "AERO_SetResolution");
		g_pAERO_DoFaceDetect = (AERO_DoHandDetect) GetProcAddress(m_hAeroTAPSDK, "AERO_DoFaceDetect");
		g_pAERO_LoadZDTable = (AERO_LoadZDTable) GetProcAddress(m_hAeroTAPSDK, "AERO_LoadZDTable");
		g_pAERO_DoAdjust = (AERO_DoAdjust) GetProcAddress(m_hAeroTAPSDK, "AERO_DoAdjust");
		g_pAERO_IsNewFrame = (AERO_IsNewFrame) GetProcAddress(m_hAeroTAPSDK, "AERO_IsNewFrame");
		g_pAERO_GetTracker = (AERO_GetTracker) GetProcAddress(m_hAeroTAPSDK, "AERO_GetTracker");
		g_pAERO_EnableGrayscale = (AERO_EnableGrayscale) GetProcAddress(m_hAeroTAPSDK, "AERO_EnableGrayscale");
		g_pAERO_DoHandDetect = (AERO_DoHandDetect) GetProcAddress(m_hAeroTAPSDK, "AERO_DoHandDetect");
		g_pAERO_GetGrayImage = (AERO_GetGrayImage) GetProcAddress(m_hAeroTAPSDK, "AERO_GetGrayImage");
		g_pAERO_GetColorDepthImage = (AERO_GetColorDepthImage) GetProcAddress(m_hAeroTAPSDK, "AERO_GetColorDepthImage");
		g_pAERO_SetDebugWnd = (AERO_SetDebugWnd) GetProcAddress(m_hAeroTAPSDK, "AERO_SetDebugWnd");

		g_pAERO_SetHandMode = (AERO_SetHandMode) GetProcAddress(m_hAeroTAPSDK, "AERO_SetHandMode");
		g_pAERO_Set2HandMode = (AERO_Set2HandMode) GetProcAddress(m_hAeroTAPSDK, "AERO_Set2HandMode");
		g_pAERO_SetSideInMode = (AERO_SetSideInMode) GetProcAddress(m_hAeroTAPSDK, "AERO_SetSideInMode");
		g_pAERO_GetZoomArea = (AERO_GetZoomArea) GetProcAddress(m_hAeroTAPSDK, "AERO_GetZoomArea");
		g_pAERO_SetNearRange = (AERO_SetNearRange) GetProcAddress(m_hAeroTAPSDK, "AERO_SetNearRange");
		g_pAERO_GetFaceObjects = ( AERO_GetFaceObjects )GetProcAddress(m_hAeroTAPSDK, "AERO_GetFaceObjects");
		g_pAERO_EnableZoomAdjust = ( AERO_EnableZoomAdjust )GetProcAddress(m_hAeroTAPSDK, "AERO_EnableZoomAdjust");
		g_pAERO_GetPeakPos = ( AERO_GetPeakPos )GetProcAddress(m_hAeroTAPSDK, "AERO_GetPeakPos");
		g_pAERO_GetColorDepthImage = (AERO_GetColorDepthImage) GetProcAddress(m_hAeroTAPSDK, "AERO_GetColorDepthImage");
		g_pAERO_GetObject = (AERO_GetObject) GetProcAddress(m_hAeroTAPSDK, "AERO_GetObject");
		g_pAERO_GetSmallImage = (AERO_GetSmallImage) GetProcAddress(m_hAeroTAPSDK, "AERO_GetSmallImage");
		g_pAERO_GetSmallImageW = (AERO_GetSmallImageW) GetProcAddress(m_hAeroTAPSDK, "AERO_GetSmallImageW");
		g_pAERO_EnableDepthLog = (AERO_EnableDepthLog) GetProcAddress(m_hAeroTAPSDK, "AERO_EnableDepthLog");


		g_pAERO_GetSDKVersion = ( AERO_GetSDKVersion )GetProcAddress(m_hAeroTAPSDK, "AERO_GetSDKVersion");
		g_pAERO_GetFWVersion = ( AERO_GetFWVersion )GetProcAddress(m_hAeroTAPSDK, "AERO_GetFWVersion");
		g_pAERO_GetSN = ( AERO_GetSN )GetProcAddress(m_hAeroTAPSDK, "AERO_GetSN");
		g_pAERO_GetPID = ( AERO_GetSN )GetProcAddress(m_hAeroTAPSDK, "AERO_GetPID");
*/


#endif